/**
 * Type-specific classes for boolean elements or keys.
 *
 * <p>
 * Not all classes are provided in a boolean-specific version: sorted sets and maps not generated,
 * as they are useless. Unsorted sets and maps are kept around for orthogonality, whereas
 * {@link it.unimi.dsi.fastutil.booleans.BooleanCollection} is used by maps with boolean values.
 */
package it.unimi.dsi.fastutil.booleans;