/**
 * Type-specific classes for object elements or keys.
 *
 * <p>
 * Whenever possible and useful, {@code fastutil} provides both typical collections, which compare
 * objects using {@code equals()}, and {@linkplain it.unimi.dsi.fastutil.objects.ReferenceSet
 * <em>reference-based</em> collections}, which use identity ({@code ==}). See the related comments
 * in the overview.
 */
package it.unimi.dsi.fastutil.objects;
